var
Commandport : word;
Dataport : word;
FIFOPort : word;
SignaturePort : word;
ResetPort : word;

{$I 4a24cons.pas}


procedure SetupPorts(theport: word);
begin
   FIFOPort := theport + FIFOPortOffset;
   CommandPort := theport + CommandPortOffset;
   DataPort := theport + DataPortOffset;
   SignaturePort := theport + SignaturePortOffset;
   ResetPort := theport + ResetPortOffset;
end;

procedure WriteParamWord(theparm: word;thedata : word);
{16 bit only parameter write}
var foo : word;
begin
  portw[DataPort] := word(thedata);
  portw[CommandPort] := (theparm or WriteFlag);
  repeat
    foo := portw[CommandPort];
  until (foo and BusyMask = 0);
end;

function ReadParamWord(theparm: word):word;
{ 16 bit parameter read }
var
foo : word;
rdata : word;
begin
  portw[CommandPort] := (theparm);
  repeat
    foo := portw[CommandPort];
  until (foo and BusyMask = 0);
  rdata := Portw[DataPort];
  ReadParamWord :=rdata;
end;

function CardThere4A24: boolean;
begin
  if portw[SignaturePort] = A24Signature then CardThere4A24 := true else CardThere4A24 := false;
end;

procedure ResetDSP;
begin
  portw[ResetPort] := $FFFF;
  portw[ResetPort] := $0000;
end;

procedure ClearFIFO;
begin
  WriteParamWord(HostFIFOClr,0);
end;

procedure SetRate(freq : real);
var
divreal : real;
divint : longint;
begin
  divreal := sysclk/freq;
  divint := trunc(divreal+0.5);
  WriteParamWord(TimerLow,word(divint and $0000FFFF));
  WriteParamWord(TimerHigh,word(divint shr 16));
end;

procedure SetBurstCount(bs : longint);
begin
  WriteParamWord(BurstCountLow,word(bs and $0000FFFF));
  WriteParamWord(BurstCountHigh,word(bs shr 16));
end;

procedure EnableTimer;
var
olddata : word;
begin
  olddata := ReadParamWord(ConvStartReg);
  WriteParamWord(ConvStartReg,olddata or CSTimerFre);
end;

procedure DisableTimer;
var
olddata : word;
begin
  olddata := ReadParamWord(ConvStartReg);
  WriteParamWord(ConvStartReg,olddata and ($FFFF xor CSTimerFre));
end;

procedure SetFIFOThreshold(setth,clrth : word);
begin
  WriteParamWord(IRQSetThresh,setth);
  WriteParamWord(IRQClrThresh,clrth);
end;

procedure SetupForPolledMode(chan,mode : byte);
var
mdata : word;
begin
  WriteParamWord(ConvStartReg,0);  { first disable any conversion sources }
  { We setup the first mux table entry with our desired channel}
  { and input mode, also setting the MTDPtrClr bit so the table pointer }
  { will not advance with start of conversion = latch behavior}
  WriteParamWord(MuxTableWAdd,0);
  WriteParamWord(MuxTablePtr,0);
  mdata := ((mode or MTDPtrClr) shl 8) or chan;
  WriteParamWord(MuxTableData,mdata);
  { enable host convert starts - disable all else }
  WriteParamWord(ConvStartReg,CSHostStart);
end;

procedure SetupForTimerStartNoScan(chan,mode : byte);
var
mdata : word;
begin
  WriteParamWord(ConvStartReg,0);  { first disable any conversion sources }
  { We setup the first mux table entry with our desired channel}
  { and input mode, also setting the MTDPtrClr bit so the table pointer }
  { will not advance with start of conversion = latch behavior}
  WriteParamWord(MuxTableWAdd,0);
  WriteParamWord(MuxTablePtr,0);
  mdata := ((mode or MTDPtrClr) shl 8) or chan;
  WriteParamWord(MuxTableData,mdata);
  { enable timer convert starts - disable all else }
  WriteParamWord(ConvStartReg,CSTimerStart);
end;

procedure SetupForTimerStartWithScan(startchan,endchan : word;mode : byte);
var
index : word;
mdata : word;
chan : word;
begin
  WriteParamWord(ConvStartReg,0);  { first disable any conversion sources }
  { We setup sequential mux table entries with our desired channel}
  { and input modes, setting the MTDPtrClr bit on the last chan}
  index := 0;
  for chan := startchan to endchan do
  begin
    mdata := ((mode) shl 8) or byte(chan and $00FF);
    if chan = endchan then mdata := ((mode or MTDPtrClr) shl 8) or byte(chan and $00FF);
    WriteParamWord(MuxTableWAdd,index);
    WriteParamWord(MuxTableData,mdata);
    index := index +1;
  end;
  { enable timer convert starts - disable all else }
  WriteParamWord(MuxTablePtr,0);
  WriteParamWord(ConvStartReg,CSTimerStart);
end;

procedure PolledModeSetChannel(chan,mode : byte);
var mdata : word;
begin
  mdata := ((mode or MTDPtrClr) shl 8) or chan;
  WriteParamWord(MuxTableWAdd,0);
  WriteParamWord(MuxTableData,mdata);
end;

function PolledModeReadAD : word;
{ simple polled startconv/poll/readad data routine }
begin
  portw[FIFOport] := 0; { start a conv }
  while (portw[CommandPort] and DAVMask) = 0 do; { wait for data avail }
  PolledModeReadAD := portw[FIFOPort]; { get the data }
end;

procedure WriteEEPROM(add,data : word);
begin
  WriteParamWord(EEPROMDataIn,data);
  WriteParamWord(EEPROMAdd,add);
  WriteParamWord(EEPROMCom,EEWriteCom);
  WriteParamWord(DOEEPROMFlag,TrueF);
  while ReadParamWord(DOEEPROMFLAG) <> 0 do;
end;

procedure WriteEnableEEPROM;
begin
  WriteParamWord(EEPROMDataIn,0);
  WriteParamWord(EEPROMAdd,$FFFF);
  WriteParamWord(EEPROMCom,EEMiscCom);
  WriteParamWord(DOEEPROMFlag,TrueF);
  while ReadParamWord(DOEEPROMFLAG) <> 0 do;
end;

function ReadEEPROM(add : word) : word;
begin
  WriteParamWord(EEPROMAdd,add);
  WriteParamWord(EEPROMCom,EEReadCom);
  WriteParamWord(DOEEPROMFlag,TrueF);
  while ReadParamWord(DOEEPROMFLAG) <> 0 do;
  ReadEEPROM := ReadParamWord(EEPROMDataOut);
end;

procedure SelectInterrupt(Int :  byte);
begin
  WriteParamWord(IRQDriveReg,int); { select IRQ without enable }
  WriteParamWord(IRQDriveReg,int or IDTSEnable or IDMask); { select IRQ with enable }
end;

function MakeWordSigned(udata: longint) : longint;
begin
 if  udata > 32767 then udata := udata -65536;
 MakeWordSigned := udata;
end;

function GainFromFS(fs,efs : real) : real;
begin
  GainFromFS := ((efs/fs) -1) *65536;
end;

function ModeString(mode,se : word) : string;
var
tstr : string;
begin
  case mode of
    CardMode2d5b : tstr := '-2.5 .. +2.5 (W4 down W5 up)';
    CardMode5b : tstr := '-5 .. +5  (W4 up W5 up)';
    CardMode5u : tstr := '0 .. +5 (W4 down W5 down)';
  end {case};
  if se = 0 then tstr := tstr +' Differential' else tstr := tstr+' Single ended';
  ModeString := tstr;
end;

function FIFOFullQ : boolean;
begin
  if portw[CommandPort] and IRQSetMask = 0 then FIFOFullQ := false else FIFOFullQ := true;
end;
