{ 4A24 DEMO SOFTWARE - Interrupt Driven 4A24 READ AND PRINT ROUTINE }


{$R+}    {Range checking }
{$S+}    {Stack checking }
{$I+}    {I/O checking }
{$N-}    {No numeric coprocessor}

{$DEFINE MAIN}                                 { to allow 4A24 low compile by itself }
uses dos,crt;
{$I 4A24LOW }                 { include low level 4A24 I/O }
{$I 8259}
{ program polled 4A24 }
const
AverageTimes = 2000;
BlockSize = 32000;

var

TheInterrupt : byte;
InterruptFlag : boolean;
VectorSave : pointer;
TheRate : longint;
DAQRecSize : word;
Index : word;
DataBuf : array[0.. blocksize -1] of word;
Channel :  byte;
ADCData : array[0..15] of longint;
RawData : longint;
RawMax,RawMin : array[0..15] of longint;
Count : word;
TheCardMode : word;
RealData,RealPP : real;
Scale : real;
Signed : boolean;
SingleFlag: boolean;
Sstring,Istring : string;
InMode,InputMode,MinChannel,Maxchannel,NChannel : byte;

procedure CheckFor4A24;
begin
  if not CardThere4A24 then
  begin
    writeln('Could not find 4A24... aborting program');
    halt(2);
  end; { if }
end; { Check for 4A24 }

procedure InterruptServiceRoutine;
interrupt;
begin
  InterruptFlag :=  true;
  if (TheInterrupt <= 7) then IssueEOI else IssueEOI2;
end; { InterruptServiceRoutine }

procedure BumOut(s : string);
begin
  DisableTimer;
  WriteParamWord(ConvStartReg,0);
  ClearFIFO;
  writeln(#7,s);
  halt(2);
end;

procedure GetParms;
var retcode : integer;
begin
  TheRate := 5000;
  if length(ParamStr(1)) > 0 then
  begin
    val(ParamStr(1),TheRate,Retcode);
  end;
  if retcode <> 0 then Bumout('Bad conversion rate');
  if TheRate < 1000 then Bumout('Too slow to wait for');
  If TheRate > 570000 then Bumout('Too fast for a 4A24-5');
  TheInterrupt := 5;
  if ParamCount > 1 then
  begin
    val(ParamStr(2),TheInterrupt,Retcode);
  end;
  if retcode <> 0 then Bumout('Bad Interrupt');
  if TheInterrupt < 2 then Bumout('Interrupt # too low');
  if TheInterrupt > 15 then Bumout('Interrupt # too high');
  if TheInterrupt = 14 then Bumout('Interrupt # conflicts with HD');
  if TheInterrupt = 6 then Bumout('Interrupt # Conflicts with Floppy');
  if TheInterrupt = 13 then Bumout('Interrupt # conflicts with NPU');
  if TheInterrupt = 8 then Bumout('Interrupt # Conflicts with RTC');

end;  { GetRate }

begin  { main }
  GetParms;
  SetupPorts(A24DefaultPort);
  CheckFor4A24;
  MinChannel := 0;
  SetupForPolledMode(0,0);
  RawData := PolledModeReadAD;
  RawData := PolledModeReadAD;
  RawData := PolledModeReadAD;
  RawData := PolledModeReadAD;
{  ResetDSP;  in case user changed jumpers since power-up
  delay(2000); this causes trouble with cardmode detection  }
  TheCardMode := ReadParamWord(cardmode);
  Signed := true;
  case TheCardMode of
    CardMode5b : Scale := 5/32767;
    CardMode2d5b : Scale := 2.5/32767;
    CardMode5u :
    begin
      Scale := 5/65535;
      Signed := false;
    end;
  end; { case }
  if signed then Sstring := 'signed' else Sstring := 'unsigned';
  if ReadParamWord(SingleEnd) <> 0 then SingleFlag := true else SingleFlag := false;
  ClrScr;
  write('4A24 Int ',TheInterrupt,' mode: ');
  writeln(ModeString(TheCardMode,ReadParamWord(SingleEnd)),' ',(Sstring),' Rate = ',TheRate);
  MaxChannel := 7;
  if SingleFlag then
  begin
    MaxChannel := 15;
  end;
  NChannel := MaxChannel+1;
  DAQRecSize := NChannel*AverageTimes;
  DisableTimer;
  ClearFIFO;
  DumpFIFO;
  SetFIFOThreshold(NChannel*AverageTimes,NChannel*AverageTimes-1);
  SetupForTimerStartWithScan(MinChannel,MaxChannel,InModeNormal);
  SetRate(TheRate);
  SelectInterrupt(TheInterrupt);
  MaskInterrupt(TheInterrupt);
  GetHardIntVec(TheInterrupt,VectorSave);
  SetHardIntVec(TheInterrupt,@InterruptServiceRoutine);
  UnMaskInterrupt(TheInterrupt);
  EnableTimer;

  while not(KeyPressed) do
  begin
    if FIFOFullQ then Bumout('FIFO full before I got there...');

    while not InterruptFlag do; { wait for a block of data }
    InterruptFlag := false;
    for Index:= 0 to DAQRecSize-1 do DataBuf[Index] := portw[FIFOPort]; { get data }

    writeln;
    GotoXY(1,3);
    for channel := 0 to MaxChannel do
    begin
      RawMin[Channel] := 100000;
      RawMax[Channel] := -100000;
      ADCData[Channel] := 0;
    end;
    for Count := 0 to AverageTimes-1 do
    begin
     for  Channel := 0 to MaxChannel do
     begin
       RawData := DataBuf[Count*(MaxChannel+1)+Channel];
       if Signed then RawData := MakeWordSigned(RawData);
       if RawData > RawMax[Channel] then RawMax[Channel] := RawData;
       if RawData < RawMin[Channel] then RawMin[Channel] := RawData;
       ADCData[Channel] := ADCData[Channel] + RawData;
     end;
   end;
   for channel := 0 to MaxChannel do
   begin
     ADCData[Channel] := ADCData[Channel] div AverageTimes;
     RealData := ADCData[Channel] * Scale;
     RealPP := (RawMax[Channel] - RawMin[Channel])*Scale;
     case InputMode of
       InModeNormal: Istring :=  '(Normal) ';
       InModeReverse: Istring := '(Reverse)';
       InModeRef: Istring :=     '(Reference)';
       InModeGnd: Istring :=     '(Ground)';
     end; {case}
     writeln(Istring,' Channel ',Channel,' Data ',ADCData[Channel]:5,'  ',RealData:1:5,'  PP AC ',RealPP:1:5,'    ');
   end; { for channel }
   writeln;
  end; { while not(KeyPressed) }
  MaskInterrupt(TheInterrupt);
  SetHardIntVec(TheInterrupt,VectorSave);
  halt(1);
end.