{4A24 SCOPE DEMO}

uses graph,crt,dos;
{$R-}                                          { for speed and some obscure bugs }
{$DEFINE MAIN}                                 { to allow 6A20 low compile by itself }
{$I 4A24low }                                  { include low level 6A20 I/O }
{ program 4A20SCOPE }


type
point = record
  X : integer;
  Y : integer;
end;

box = record
 Start : point;
 Size : point;
end;

title = string[10];

labeledbox = record
  TitleBox : box;
  ValueBox : box;
  Title : string;
end;

scopemodetype = (Manual,Single,Auto,Normal);
modetitlestype = array[Manual..Normal] of string;
menuindextype =  (MODE,VDIV,TDIV,TRIG,VOFF,TOFF,RLEN,CHAN);
menuboxestype = array[MODE..CHAN] of labeledbox;
menutitlestype = array[MODE..CHAN] of title;
menuspeed = (DownFast,DownSlow,UpFast,UpSlow,NoChange);

scopesettingstype = record
  ScopeMode : scopemodetype;                   { main scope mode }
  VoltsDivIndex : integer;                     { index into array of uvolts per division settings }
  TimesDivIndex : integer;                     { index into array of useconds per division settings }
  VoltOff : integer;                           { vertical divisions of offset }
  TimeOff : integer;                           { horizontal divisions of offset }
  TriggerOff :integer;                         { offset in samples from start to trigger event }
  TriggerPoint : integer;                      { the trigger point setting (in ADcounts)}
  TriggerPlus : boolean;                       { the trigger polarity (true if positive edge triggered)}
  RecordLenIndex : integer;                    { index into array of record lengths (in samples)}
  Channel : integer;                           { the current channel }
end;

const
TimeSettings = 11;
VoltSettings = 12;
RecordSettings = 6;
MaxRecordLength = 20000;

type
timesperdivtype = array [1..TimeSettings] of longint;  { in microseconds }
voltsperdivtype = array [1..VoltSettings] of longint;  { in microvolts }
recordlengthtype = array [1..RecordSettings] of integer;   { in samples }


const
A24Card = $220;                                { default 4A24 location }
BufferLength       = MaxRecordLength;          { about 20K samples (40K bytes) max }
RateClock          = 50000000;                 { all 4A24s use 50 Mhz clock }
SamplesPerDivision = 40;
Hdivs              = 10;
SamplesPerScreen   = SamplesPerDivision*HDivs;
Vdivs              = 8;
MaxChannel         = 7;
MinChannel         = 0;
FirstMode          = Manual;
LastMode           = Normal;

ModeTitles : modetitlestype = ('Manual','Single','Auto','Normal');
MenuTitles : menutitlestype = ('MODE','V/DIV','T/DIV','TRIG','V/OFF','T/OFF','REC LEN','CHAN');
TimesPerDiv : timesperdivtype = (500,1000,2000,5000,10000,20000,50000,100000,200000,500000,1000000);
VoltsPerDiv : voltsperdivtype = (200,500,1000,2000,5000,10000,20000,50000,100000,200000,500000,1000000);
RecordLengths : recordlengthtype = (400,1000,2000,4000,10000,20000);
FirstMenuIndex = MODE;
LastMenuIndex = CHAN;

var
Signed : boolean;
VoltsPerCount : real;
ScreenBox : labeledbox;
MainBox : labeledbox;
VariablesBox : labeledbox;
ParameterBox : labeledbox;
ScopeScreen : labeledbox;
MessageBox : labeledbox;

MenuIndex : menuindextype;
MenuBoxes : menuboxestype;
GraphDriver : integer;
BoxColor : word;
TitleColor : word;
ValueTextColor : word;
TitleTextColor : word;
ValueColor : word;
DataColor : word;
GratColor : word;
BackColor : word;
PointerColor : word;
ArrowSize : word;

ScopeSettings : ScopeSettingsType;
HorPixelsPerDivision : integer;
VerPixelsPerDivision : integer;
VectorSave : pointer;
TheCardMode : word;
DataBuffer : array[1..BufferLength] of integer;

procedure BoxInBox(refBox : labeledbox; var newBox : labeledbox;
                   begX : real; begY : real; endX : real; endY : real);
var
sDelX : integer;
sDelY : integer;
eDelX : integer;
eDelY : integer;
begin
  with refBox.ValueBox do
  begin
    sDelX := round(begX * Size.X);
    sDelY := round(begY * Size.Y);
    eDelX := round((1-endX) * Size.X);
    eDelY := round((1-endY) * Size.Y);
    newBox.ValueBox.Start.X := Start.X + sDelX;
    newBox.ValueBox.Start.Y := Start.Y + sDelY;
    newBox.ValueBox.Size.X := Size.X - (sDelX + eDelX);
    newBox.ValueBox.Size.Y := Size.Y - (sDelY + eDelY);
  end;
end;

procedure FillBox(aBox : box; pattern : word; color : word);
begin
  SetFillStyle(pattern,color);
  with aBox do
  Bar(Start.X +1,Start.Y+ 1,Start.X + Size.X-1,Start.y + Size.y-1);
end;

procedure CenterOfBox(theBox : box; var center : point);
begin
  center.X := theBox.Start.X + (theBox.Size.X div 2);
  center.Y := theBox.Start.Y + (theBox.Size.Y div 2);
end;

procedure DrawBox(theBox : box);
begin
  with theBox do
    rectangle(Start.X,Start.Y,Start.X+Size.X,Start.Y+Size.Y);
end;

procedure CenterTextInBox(theBox : box; text : string);
var
center : point;
begin
  CenterOfBox(theBox,center);
  SetTextJustify(CenterText,CenterText);
  OutTextXY(center.X,center.Y+1,text);
end;

procedure WriteTextValue(theBox : labeledBox; value : string);
begin
  FillBox(theBox.ValueBox,SolidFill,ValueColor);
  SetColor(ValueTextColor);
  CenterTextInBox(theBox.ValueBox,value);
end;

procedure ClearValueBox(theBox : labeledBox; color : word);
begin
  FillBox(theBox.ValueBox,SolidFill,color);
end;

procedure WriteTitle(theBox : labeledbox; title : string);
begin
  FillBox(theBox.TitleBox,SolidFill,TitleColor);
  SetColor(TitleTextColor);
  CenterTextInBox(theBox.TitleBox,title);
end;

procedure WriteNumberValue(var theBox : labeledBox; value : longint);
var
textValue : string;
begin
  Str(value,textValue);
  WriteTextValue(theBox,textValue);
end;


procedure MakeLabeledBox(var theBox : labeledBox; theTitle : string);
var
theTextHeight : word;
theMaxTextWidth : word;
textInfo : TextSettingsType;
begin
  GetTextSettings(textInfo);
  theTextHeight := textInfo.CharSize * 8;
  with theBox do
  begin
    TitleBox.Start.X := ValueBox.Start.X;
    TitleBox.Start.Y := ValueBox.Start.Y;
    TitleBox.Size.X := ValueBox.Size.X ;
    TitleBox.Size.Y := theTextHeight + 2;
    ValueBox.Start.Y := ValueBox.Start.Y + TitleBox.Size.Y;
    ValueBox.Size.Y := ValueBox.Size.Y - TitleBox.Size.Y;
  end;
  WriteTitle(theBox,theTitle);
  SetColor(BoxColor);
  DrawBox(theBox.TitleBox);
  DrawBox(theBox.ValueBox);
end;

procedure InitColors;
begin
  if GraphDriver <> HercMono then
  begin
    BoxColor := EGAWhite;
    TitleColor := EGAWhite;
    ValueTextColor := EGAYellow;
    ValueColor := EGABlue;
    TitleTextColor := EGABlack;
    DataColor := EGALightGreen;
    GratColor := EGAGreen;
    BackColor := EGADarkGray;
    PointerColor := EGAWhite;
  end
  else
  begin
    BoxColor := White;
    TitleColor := White;
    ValueTextColor := White;
    ValueColor := Black;
    TitleTextColor := Black;
    DataColor := White;
    GratColor := White;
    BackColor := Black;
    PointerColor := White;
  end;
end;

procedure InitMenu;
var
begX,begY,endX,endY,stepY,sizeY : real;
menuIndex : menuIndextype;
begin
  begX := 0;
  endX := 1;
  begY := 0;
  stepY := 0.101;
  sizeY := 0.08;
  for menuIndex := FirstMenuIndex to LastMenuIndex do
  begin
    BoxInBox(VariablesBox,MenuBoxes[menuIndex],begX,begY,endX,begY+sizeY);
    MakeLabeledBox(MenuBoxes[menuIndex],MenuTitles[menuIndex]);
    begY := begY + stepY
  end;
end;

procedure Arrow(x : word; y : word; color : word);
begin
  Setcolor(color);
  Line(x-ArrowSize,y,x,y);
  Line(x-ArrowSize div 4 ,y+ArrowSize div 4 ,x,y);
  Line(x-ArrowSize div 4 ,y-ArrowSize div 4 ,x,y);
end;

procedure ArrowAtBox(theBox : labeledbox; color : word);
var
xArrow,yArrow : word;
begin
  with theBox.ValueBox do
  begin
    xArrow := Start.X-1;
    yArrow := Start.Y;
  end;
  Arrow(xArrow,yArrow,color);
end;

procedure MarkBox(theBox : Labeledbox);
begin
  ArrowAtBox(theBox,PointerColor);
end;

procedure UnMarkBox(theBox : Labeledbox);
begin
  ArrowAtBox(theBox,BackColor);
end;

procedure MenuIndexUp;
begin
  UnMarkBox(MenuBoxes[MenuIndex]);
  if MenuIndex > FirstMenuIndex then MenuIndex := Pred(MenuIndex);
  MarkBox(MenuBoxes[MenuIndex]);
end;

procedure MenuIndexDown;
begin
  UnMarkBox(MenuBoxes[MenuIndex]);
  if MenuIndex < LastMenuIndex then MenuIndex := Succ(MenuIndex);
  MarkBox(MenuBoxes[MenuIndex]);
end;


procedure InitScreen;
var size : point;
var origin : point;
var aBox : box;
begin
  with ScreenBox.ValueBox do
  begin
    Start.X :=0;
    Start.Y :=0;
    Size.X :=GetMaxX;
    Size.Y :=GetMaxY;
    ArrowSize := Size.X div 25;
    end;
  BoxInBox(ScreenBox,MainBox,0.02,0.02,0.98,0.98);
  MakeLabeledbox(MainBox,'4A24 SCOPE DEMO');
  ClearValueBox(MainBox,BackColor);
  BoxInBox(MainBox,VariablesBox,0.05,0.05,0.2,0.98);
  BoxInBox(MainBox,ScopeScreen,0.25,0.05,0.95,0.85);
  MakeLabeledBox(ScopeScreen,'Data Display');
  BoxInBox(MainBox,MessageBox,0.25,0.9,0.95,0.98);
  MakeLabeledBox(MessageBox,'Messages');
  WriteTextValue(MessageBox,'Hello world');
  InitMenu;
  MenuIndex := FirstMenuIndex;
  MenuIndexUp;                                 { forces refresh of arrow }
end;


procedure BumpIndex(var index : integer; limit : integer; bump: menuspeed);
begin
  case bump of
    UpFast,UpSlow :
    begin
      if index < limit then index := index +1;
    end;
    DownFast,DownSlow :
    begin
      if index > 1 then index := index -1;
    end;
  end; { case }
end;

function GetMaxVoffset : integer;
var
inputrange : longint;
offsetrange : longint;
begin
  with ScopeSettings do
  begin
    inputrange := Vdivs * VoltsPerDiv[VoltSettings];   { assuming volts per division at 1 is max }
    offsetrange := inputrange - (Vdivs * VoltsPerDiv[VoltsDivIndex]);
    offsetrange := offsetrange div VoltsPerDiv[VoltsDivIndex];
    offsetrange := offsetrange div 2;
  end;{with}
  GetMaxVoffset := offsetrange;
end;

procedure VoltUnitString(uvolts : longint; var vstring : string);
var
nstring : string;
begin
  case uvolts div 100 of
    2..5 :
    begin
      str(uvolts,nstring);
      nstring := nstring + ' uV';
    end;
    10..5000 :
    begin
      uvolts := uvolts div 1000;
      str(uvolts,nstring);
      nstring := nstring + ' mV';
    end;
    10000 :
    begin
      uvolts := uvolts div 1000000;
      str(uvolts,nstring);
      nstring := nstring + ' V';
    end;
  end; {case}
  vstring := nstring;
end;

procedure TimeUnitString(usecs : longint ; var tstring : string);
var
nstring : string;
begin
  case usecs div 100 of
    5 :
    begin
      str(usecs,nstring);
      nstring := nstring + ' uS';
    end;
    10..5000 :
    begin
      usecs := usecs div 1000;
      str(usecs,nstring);
      nstring := nstring + ' mS';
    end;
    10000 :
    begin
      usecs := usecs div 1000000;
      str(usecs,nstring);
      nstring := nstring + ' S';
    end;
  end; {case}
  Tstring := nstring;
end;


function InitGraphics : boolean;
var
GraphMode : integer;
ErrorCode : integer;
x : integer;
y : integer;
begin
  GraphDriver := Detect;
  InitGraph(GraphDriver,GraphMode,'');
  ErrorCode := GraphResult;
  If ErrorCode <> GrOk then
  begin
    writeln('Undetermined problem with graphics... aborting progran');
    halt(2);
  end;
end;

procedure InitScopeScreen;
var
hindexr : real;
vindexr : real;
hindex : word;
vindex : word;
hstep : real;
vstep : real;

begin
  ClearValueBox(ScopeScreen,ValueColor);
  WriteTextValue(MessageBox,' ');
  with ScopeScreen.ValueBox do
  begin
    SetViewPort(Start.X,Start.Y,Start.X+Size.X,Start.Y+Size.Y,ClipOn);
    hstep := (Size.X /Hdivs);
    vstep := (Size.Y /Vdivs);
    hindexr := 0;
    hindex := Round(hindexr);
    Setcolor(GratColor);
    SetLineStyle(UserBitLn,$8888,NormWidth);
    while hindex < Size.X do
    begin
      hindex := Round(hindexr);
      Line(hindex,1,hindex,Size.Y-1);
      hindexr := hindexr + hstep;
    end{ while hindex };
    vindexr := 0;
    vindex := Round(vindexr);
    while vindex < Size.Y do
    begin
      vindex := Round(vindexr);
      Line(1,vindex,Size.X-1,vindex);
      vindexr := vindexr + vstep;
    end {while vindex};
  end;
  SetLineStyle(UserBitLn,$FFFF,NormWidth);
  SetViewPort(0,0,GetMaxX,GetMaxY,ClipOn);
  SetColor(BoxColor);
  DrawBox(ScopeScreen.ValueBox);
  WriteTextValue(MessageBox,'ready');
end;

procedure CheckFor4A24;
begin
  if not CardThere4A24 then
  begin
    writeln('Could not find 4A24... aborting program');
    halt(2);
  end; { if }
end; { Check for 4A24 }


procedure Initialize4A24;
var
nString : string;
begin
  SetupPorts(A24DefaultPort);
  CheckFor4A24;
  WriteTextValue(MessageBox,'4A24 ok... proceeding with set-up');
  ResetDSP; { in case user changed jumpers }
  delay(500);
  SetupForPolledMode(0,0);
  TheCardMode := ReadParamWord(cardmode);
  Signed := true;
  case TheCardMode of
    CardMode5b : VoltsPerCount := 5/32767;
    CardMode2d5b : VoltsPerCount := 2.5/32767;
    CardMode5u :
    begin
      VoltsPerCount := 5/65535;
      Signed := false;
    end;
  end; { case }
end;

procedure SelectSampleRate;
var
usecsperdivision : longint;
sampleperiod : real;
begin
  usecsperdivision := TimesPerDiv[ScopeSettings.TimesDivIndex];
  sampleperiod := usecsperdivision/SamplesPerDivision;
  SetPeriod(sampleperiod/1e6);
end;

Procedure AcquireData;
var
rlength: longint;
index,startmask,fifocnt : word;
tstring : string;
begin
  rlength := RecordLengths[ScopeSettings.RecordLenIndex];
  str(rlength,tstring);
  WriteTextValue(MessageBox,'acquiring '+ tstring+' words');
  if ScopeSettings.ScopeMode = Single then
  begin
    startmask := CSTimerStart or CSExtStartBurst or CSExtStartBurstPol;
  end
  else
  begin
    startmask := CSTimerStart or CSHostStartBurst;
  end;
  WriteParamWord(ConvStartReg,startmask);
  SetBurstCount(rlength);
  if ScopeSettings.ScopeMode = Manual then
  begin
    portw[FIFOPort] := 0;
  end;
  while not BurstCountZ do { nothing };
  fifocnt := ReadParamWord(BoundedFifoCnt);
  str(fifocnt,tstring);
  WriteTextValue(MessageBox,'FIFO data count = '+tstring);
  for index := 1 to rlength do
  begin
    DataBuffer[index] := portw[FIFOPort];
  end;
end;

function GetYScale(screenht : integer) : real;
var
uvoltsperdiv : longint;
uvoltspercount : real;
pixelsperdiv : real;

begin
  with ScopeSettings do
  begin
    uvoltsperdiv := VoltsPerDiv[VoltsDivIndex];
    pixelsperdiv := screenht / Vdivs;              { Vdivs vertical divisions }
    uvoltspercount := VoltsPerCount*1e6;
    GetYScale := (pixelsperdiv*uvoltspercount)/uvoltsperdiv
  end;
end;

procedure ScanTillAbove(triggerpoint : integer; var tindex : integer);
var
index : integer;
lastrec : integer;
begin
  lastrec := RecordLengths[ScopeSettings.RecordLenIndex];
  index := 0;
  while (DataBuffer[index] <= triggerpoint) or (index = lastrec) do  index := index + 1;
  tindex := index;
end;

procedure ScanTillBelow(triggerpoint : integer; var tindex : integer);
var
index : integer;
lastrec : integer;
begin
  lastrec := RecordLengths[ScopeSettings.RecordLenIndex];
  index := 0;
  while (DataBuffer[index] >= triggerpoint) or (index = lastrec) do  index := index + 1;
  tindex := index;
end;



function GetTriggerIndex : integer;            { returns index of trigger event }
var
countsperdiv : integer;
scaledtrigger : integer;
index : integer;
triggerindex : integer;

begin
  with ScopeSettings do
  begin
    countsperdiv := VoltsPerDiv[VoltsDivIndex] div round(VoltsPerCount*1e6);
    scaledtrigger := TriggerPoint * countsperdiv;
    case TriggerPlus of
      True :
      begin
        ScanTillBelow(scaledtrigger,index);
        ScanTillAbove(scaledtrigger,index);
      end;
      False :
      begin
        ScanTillAbove(scaledtrigger,index);
        ScanTillBelow(scaledtrigger,index);
      end;
    end; {case TriggerPlus}
  end; {with ScopeSettings}
end; {GetTriggerIndex}

procedure DisplayData;
var
index : word;
xscale : real;
indexoff : integer;
yscale : real;
ydisplay : integer;
ydoff : integer;
xpos : integer;
ypos : integer;

begin
  Setcolor(DataColor);
  SetLineStyle(UserBitLn,$FFFF,NormWidth);
  with ScopeScreen.ValueBox,ScopeSettings  do
  begin
    SetViewPort(Start.X,Start.Y,Start.X+Size.X,Start.Y+Size.Y,ClipOn);
    indexoff := TimeOff * SamplesPerDivision + TriggerOff;
    ydoff := Round(Size.Y/2 - (Size.Y/Vdivs) * ScopeSettings.VoltOff);
    xscale := Size.X/(SamplesPerDivision*Hdivs);  { Hdivs divisions }
    yscale := GetYScale(Size.Y);
    index := 0;
    ydisplay := DataBuffer[index + indexoff];
    ypos := ydoff - Round((ydisplay) * yscale);
    xpos := Round(index * xscale);
    MoveTo(xpos,ypos);
    for index := 1 to  samplesperdivision * Hdivs do
    begin
      ydisplay := DataBuffer[index + indexoff];
      ypos := ydoff - Round((ydisplay) * yscale) ;
      xpos := Round(index * xscale);
      LineTo(xpos,ypos);
    end;
  end;
  SetViewPort(0,0,GetMaxX,GetMaxY,ClipOn);
end;

procedure GetAndShowData;
begin
  AcquireData;
  DisplayData;
  WriteTextValue(MessageBox,'ready');
end;

procedure BumpTimeOff(bump : menuspeed);
var
maxtoff : integer;
begin
  with ScopeSettings do
  begin
    maxtoff := RecordLengths[RecordLenIndex] - (SamplesPerDivision * Hdivs);
    maxtoff := maxtoff - TriggerOff;
    maxtoff := maxtoff div SamplesPerDivision;
    case bump of
      UpSlow : TimeOff := TimeOff + 1;
      UpFast : TimeOff := TimeOff + (Hdivs);
      DownSlow : TimeOff := TimeOff - 1;
      DownFast : TimeOff := TimeOff - (Hdivs);
    end; {case}
    if TimeOff > maxtoff then TimeOff := maxtoff;
    if TimeOff < 0 then TimeOff := 0;
  end; {with}
end;


procedure BumpVoltOff(bump : menuspeed);
var
maxvoff : integer;
minvoff : integer;
begin
  maxvoff := GetMaxVoffset;
  minvoff := maxvoff * -1;
  with ScopeSettings do
  begin
    case bump of
      UpSlow : VoltOff := VoltOff + 1;
      UpFast : VoltOff := VoltOff + (Vdivs);
      DownSlow : VoltOff := VoltOff - 1;
      DownFast : VoltOff := VoltOff - (Vdivs);
    end; {case}
    if VoltOff > maxvoff then VoltOff := maxvoff;
    if VoltOff < minvoff then VoltOff := minvoff;
  end; {with}
end;

procedure BumpMode(bump : menuspeed);
begin
  with ScopeSettings do
  begin
    case bump of
      UpSlow,UpFast :
      begin
        if ScopeMode < LastMode then ScopeMode := Succ(ScopeMode);
      end;
      DownSlow,DownFast :
      begin
        if ScopeMode > FirstMode then ScopeMode := Pred(ScopeMode);
      end;
    end; {case}
  end; {with}
end;

procedure BumpChannel(bump : menuspeed);
begin
  with ScopeSettings do
  begin
    case bump of
      UpSlow,UpFast :     Channel := Channel + 1;
      DownSlow,DownFast : Channel := Channel - 1;
    end; {case}
    if Channel > MaxChannel then Channel := MaxChannel;
    if Channel < MinChannel then Channel := MinChannel;
  end; {with}
end;

Procedure UpdateMode(bump : menuspeed);
var
nstring : string;
oldmode : scopemodetype;
begin
  with ScopeSettings do
  begin
    oldmode := ScopeMode;
    BumpMode(bump);
    nstring := ModeTitles[ScopeMode];
    WriteTextValue(MenuBoxes[MODE],nstring);
    if (bump <> NoChange) and (oldmode <> Scopemode) then
    begin
{      case ScopeMode of}

    end;
  end; {with}
end;

Procedure UpdateVOFF(bump : menuspeed);
var
nstring : string;
oldvoltoff : integer;
begin
  with ScopeSettings do
  begin
    oldvoltoff := VoltOff;
    BumpVoltoff(bump);
    Str(VoltOff,nstring);
    nstring := nstring + ' Divs';
    WriteTextValue(MenuBoxes[VOFF],nstring);
    if (bump <> nochange) and (VoltOff <> oldvoltoff) then
    begin
      InitScopeScreen;
      DisplayData;
    end;
  end;
end;

Procedure UpdateVDIV(Bump : menuspeed);
var vstring : string ;
begin
  with ScopeSettings do
  begin
    BumpIndex(VoltsDivIndex,VoltSettings,bump);
    VoltUnitString(VoltsPerDiv[VoltsDivIndex],vstring);
    WriteTextValue(MenuBoxes[VDIV],vstring);
    VoltOff := 0;
    UpDateVoff(NoChange);
  end;
end;

Procedure UpdateTDIV(Bump : menuspeed);
var
tstring : string;
begin
  with ScopeSettings do
  begin
    BumpIndex(TimesDivIndex,TimeSettings,bump);
    TimeUnitString(TimesPerDiv[TimesDivIndex],tstring);
    WriteTextValue(MenuBoxes[TDIV],tstring);
    SelectSampleRate;
  end;
end;

Procedure UpdateTOFF(Bump : menuspeed);
var
nstring : string;
oldtimeoff : integer;
begin
  with ScopeSettings do
  begin
    oldtimeoff := TimeOff;
    BumpTimeoff(bump);
    Str(TimeOff,nstring);
    nstring := nstring + ' Divs';
    WriteTextValue(MenuBoxes[TOFF],nstring);
    if (bump <> nochange) and (TimeOff <> oldtimeoff) then
    begin
      InitScopeScreen;
      DisplayData;
    end;
  end;
end;

Procedure UpdateRLEN(Bump : menuspeed);
var
screens: integer;
nstring : string;
begin
  with ScopeSettings do
  begin
    BumpIndex(RecordLenIndex,RecordSettings,bump);
    screens := RecordLengths[RecordLenIndex] div (SamplesPerDivision *Hdivs);
    Str(screens,nstring);
    nstring := nstring + ' Scrn(s)';
    WriteTextValue(MenuBoxes[RLEN],nstring);
    TimeOff := 0;
    UpdateTOFF(NoChange);
  end;
end;


Procedure UpdateCHAN(bump : menuspeed);
begin
  with ScopeSettings do
  begin
    BumpChannel(bump);
    WriteNumberValue(MenuBoxes[CHAN],Channel);
  end;
end;

procedure InitScopeParameters;
begin
  with ScopeSettings do
  begin
    ScopeMode := FirstMode;
    VoltsDivIndex := VoltSettings;
    TimesDivIndex := 1;
    VoltOff := 0;
    TimeOff := 0;
    RecordLenIndex := 1;
    Channel := 0;
    TriggerPlus := True;
    TriggerOff := 0;
    TriggerPoint :=0;
  end; {with}
end;


procedure RefreshDisplayNumbers;
begin
  with ScopeSettings do
  begin
    UpdateMODE(NoChange);
    UpdateVDIV(NoChange);
    UpdateTDIV(NoChange);
    UpdateRLEN(NoChange);
    UpdateVOFF(NoChange);
    UpdateTOFF(NoChange);
    UpdateCHAN(NoChange);
  end;
end;


procedure ReInitParameters;
begin
  InitScopeParameters;
  RefreshDisplayNumbers;
end;

procedure InitOffsets;
begin
  With ScopeSettings do
  begin
    TimeOff := 0;
    VoltOff := 0;
    RefreshDisplayNumbers;
    InitScopeScreen;
    DisplayData;
  end;
end;

procedure UpdateScopeParameter(bump : menuspeed);
begin
  case MenuIndex of
    MODE :  UpdateMODE(bump);
    VDIV :  UpdateVDIV(bump);
    TDIV :  UpdateTDIV(bump);
    VOFF :  UpdateVOFF(bump);
    TOFF :  UpdateTOFF(bump);
    RLEN :  UpdateRLEN(bump);
    CHAN :  UpdateCHAN(bump);
  end; {case}
end;

procedure KeyLoop;
var
keyChar : char;
done : boolean;

begin
  done := false;
  repeat
    KeyChar := ReadKey;
    case KeyChar of
      #0 :
      begin
        KeyChar := ReadKey;
        case KeyChar of
          #75  : UpdateScopeParameter(DownSlow);
          #79  : UpdateScopeParameter(DownFast);
          #81  : UpdateScopeParameter(UpFast);
          #77  : UpdateScopeParameter(UpSlow);
          #82  : GetAndShowData;
          #83  : InitScopeScreen;
          #71  : InitOffsets;
          #119 : ReInitParameters;
          #72  : MenuIndexUp;
          #80  : MenuIndexDown;
          #45  : done := true;
        end; {function key case}
      end; {any key case}
    end;
  until done = true;
end;


procedure ShutDown;
begin
  CloseGraph;
end;



{*****************************************************************************}


begin                                          {main}
  if not InitGraphics then
  begin
    writeln('problem with graphics... aborting');
    halt(2);
  end;
  InitColors;
  InitScreen;
  Initialize4A24;
  InitScopeParameters;
  RefreshDisplayNumbers;
  InitScopeScreen;
  KeyLoop;
  Shutdown;
  writeln('that''s all folks...');
end.
